// generated by Fast Light User Interface Designer (fluid) version 1.0303

#include "WidgetPDialUI.h"
// WidgetPDial.cc
// Original ZynAddSubFX author Nasca Octavian Paul
// Copyright (C) 2002-2005 Nasca Octavian Paul
// Copyright 2009-2010, Alan Calvert
// Copyright 2016 Will Godfrey

// This file is part of yoshimi, which is free software: you can redistribute
// it and/or modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.

// yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.   See the GNU General Public License (version 2 or
// later) for more details.

// You should have received a copy of the GNU General Public License along with
// yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
// Street, Fifth Floor, Boston, MA  02110-1301, USA.

// This file is a derivative of the ZynAddSubFX original, modified October 2010

#include <FL/fl_draw.H>
    #include <FL/Fl_Tooltip.H>
    #include <FL/Fl_Group.H>
    #include <FL/x.H>
    #include <cairo.h>
    #include <cairo-xlib.h>

TipWin::TipWin():Fl_Menu_Window(1,1) {
  tip.clear();
          graphicsType = VC_plainValue;
          set_override();
          end();
}

void TipWin::draw() {
  draw_box(FL_BORDER_BOX, 0, 0, w(), h(), Fl_Color(175));
          fl_color(FL_BLACK);
          fl_font(labelfont(), labelsize());
          if(textmode)
              fl_draw(textWithValue.c_str(), 3, 3, w()-6, textH-6,
                      Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_WRAP));
          else
              fl_draw(tip.c_str(), 3, 3, w()-6, textH-6,
                      Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_WRAP));
          if(graphicsType != VC_plainValue)
              custom_graphics(graphicsType,Pvalue,w(),h());
}

void TipWin::value(float f) {
  Pvalue=f;
}

void TipWin::customValue(string s) {
  tip.clear();
          tip = s;
          textmode = false;
          updateToolTips();
}

void TipWin::setText(string c) {
  text = c;
          textmode = true;
          updateToolTips();
}

void TipWin::setTextmode() {
  textmode=true;
          updateToolTips();
}

void TipWin::setGraphicsValueType(ValueType gv_) {
  graphicsType = gv_;
}

void TipWin::updateToolTips() {
  // combine tooltip text and value to show both
       textWithValue = text;
       textWithValue+="\n";
       textWithValue+=tip;
       // Recalc size of window
       fl_font(labelfont(), labelsize());
       int W = 260, H = 0;
       if (textmode == true)
       {  // show tip and value
          // make sure we have space for graphics if needed
          int tipW = 260, tipH = 0;
          fl_measure(text.c_str(), tipW, tipH, 0);
          tipH += custom_graph_size(graphicsType);
          fl_measure(textWithValue.c_str(), W, H, 0);
          textH = H;
          H = (tipH > H) ? (tipH) : (H);
       }
       else // show value only
       {
          fl_measure(tip.c_str(), W, H, 0);
          int gH = custom_graph_size(graphicsType);
          H = (gH > H) ? (gH) : (H);
          textH = H;
          W += gH;
       }
       W += 8;
       size(W, H);
       redraw();
}

WidgetPDial::WidgetPDial(int x,int y, int w, int h, const char *label):Fl_Dial(x,y,w,h,label) {
  //
  	//callback(value_cb, (void*)this);
          Fl_Group *save = Fl_Group::current();
          tipwin = new TipWin();
          tipwin->hide();
          Fl_Group::current(save);
          oldvalue = 0.0;
          pos = false;
          valueType = VC_plainValue;
          textset = false;
}

void WidgetPDial::init(float home_ ) {
  home = home_;
}

WidgetPDial::~WidgetPDial() {
  delete tipwin;
}

int WidgetPDial::handle(int event) {
  double dragsize, v, min = minimum(), max = maximum();
          int my;
          int mx;
  
          switch (event)
          {
              case FL_PUSH:
                  if (home > -0.5f && Fl::event_button() == 3)
          	{
              	    value(home);
              	    value_damage();
              	    if (this->when() != 0)
                           do_callback();
                      return 1;
          	}
          	if (this->when() != 0) // only effects knobs  need this
                           do_callback();
  
                  oldvalue = value();
              case FL_DRAG:
                   if(!pos)
                   {
                       tipwin->position(Fl::event_x_root(), Fl::event_y_root() + 20);
                       pos = true;
                   }
                   tipwin->show();
                   my = -((Fl::event_y() - y()) * 2 - h());
                   mx = ((Fl::event_x() - x()) * 2 - w());
                   my = (my + mx);
  
                   dragsize = 200.0;
                   if(Fl::event_state(FL_CTRL) != 0)
                       dragsize *= 10;
  		 else if (Fl::event_button() == 2)
  		     dragsize *= 3;
  		 if (Fl::event_button() != 3)
  		 {
  		     v = oldvalue + my / dragsize * (max - min);
                       v = clamp(v);
                       //printf("%d   %g    %g\n",my,v,oldvalue);
                       value(v);
                       value_damage();
                       if (this->when() != 0)
                           do_callback();
                   }
                   return 1;
                   break;
              case FL_MOUSEWHEEL:
                  if (Fl::belowmouse() != this)
                      return 1;
                  my = - Fl::event_dy();
                  dragsize = 25.0f;
                  if(Fl::event_state(FL_CTRL) != 0)
                      dragsize *= 10;
                  value(limit(value() + my / dragsize * (max - min), min, max));
                  tipwin->show();
                  value_damage();
                  if(this->when() != 0)
                      do_callback();
                  return 1;
                  break;
              case FL_ENTER:
                  {
                       if(!pos)
                       {
                           tipwin->position(Fl::event_x_root(), Fl::event_y_root() + 20);
                           pos = true;
                       }
                       if(textset)
                       {
                           tipwin->setTextmode();
                       }
                       tipwin->show();
                       return 1;
                   }
                   break;
              case FL_HIDE:
              case FL_LEAVE:
                   tipwin->hide();
                   pos = false;
                   break;
              case FL_RELEASE:
                   tipwin->hide();
                   pos = false;
                   if (this->when() == 0)
                       do_callback();
                   return 1;
                   break;
          }
          return 0;
}

void WidgetPDial::drawgradient(int cx,int cy,int sx,double m1,double m2) {
  for (int i = (int)(m1 * sx); i < (int)(m2 * sx); ++i)
          {
              double tmp = 1.0 - powf( i * 1.0 / sx, 2.0);
              pdialcolor(140 + (int) (tmp * 90), 140 + (int)(tmp * 90), 140 + (int)(tmp * 100));
              fl_arc(cx + sx / 2 - i / 2, cy + sx / 2 - i / 2, i, i, 0, 360);
          }
}

void WidgetPDial::draw() {
  int cx = x(), cy = y(), sx = w(), sy = h();
          double d = (sx>sy)?sy:sx; // d = the smallest side -2
          double dh = d/2;
          fl_color(170,170,170);
          fl_pie(cx - 2, cy - 2, d + 4, d + 4, 0, 360);
          double val = (value() - minimum()) / (maximum() - minimum());
          cairo_t *cr;
          cairo_surface_t* Xsurface = cairo_xlib_surface_create(fl_display, fl_window, fl_visual->visual,Fl_Window::current()->w(), Fl_Window::current()->h());
          cr = cairo_create (Xsurface);
          cairo_translate(cr,cx+dh,cy+dh);
          //relative lenghts of the various parts:
          double rCint = 10.5/35;
          double rCout = 13.0/35;
          double rHand = 8.0/35;
          double rGear = 15.0/35;
          //drawing base dark circle
          if (active_r()) {
              cairo_pattern_create_rgb(51.0/255,51.0/255,51.0/255);
          } else {
              cairo_set_source_rgb(cr,0.4,0.4,0.4);
          }
          cairo_arc(cr,0,0,dh,0,2*M_PI);
          cairo_fill(cr);
          cairo_pattern_t* pat;
          //drawing the inner circle:
          pat = cairo_pattern_create_linear(0.5*dh,0.5*dh,0,-0.5*dh);
          cairo_pattern_add_color_stop_rgba (pat, 0, 0.8*186.0/255, 0.8*198.0/255, 0.8*211.0/255, 1);
          cairo_pattern_add_color_stop_rgba (pat, 1, 231.0/255, 235.0/255, 239.0/255, 1);
          cairo_set_source (cr, pat);
          cairo_arc(cr,0,0,d*rCout,0,2*M_PI);
          cairo_fill(cr);
          //drawing the outer circle:
          pat = cairo_pattern_create_radial(2.0/35*d,6.0/35*d,2.0/35*d,0,0,d*rCint);
          cairo_pattern_add_color_stop_rgba (pat, 0, 231.0/255, 235.0/255, 239.0/255, 1);
          cairo_pattern_add_color_stop_rgba (pat, 1, 186.0/255, 198.0/255, 211.0/255, 1);
          cairo_set_source (cr, pat);
          cairo_arc(cr,0,0,d*rCint,0,2*M_PI);
          cairo_fill(cr);
          //drawing the "light" circle:
          if (active_r()) {
              cairo_set_source_rgb(cr,0,197.0/255,245.0/255); //light blue
          } else {
              cairo_set_source_rgb(cr,0.6,0.7,0.8);
          }
          cairo_set_line_width (cr, 2);
          cairo_new_sub_path(cr);
          cairo_arc(cr,0,0,d*rGear,0.75*M_PI,+val*1.5*M_PI+0.75*M_PI);
          cairo_stroke(cr);
          //drawing the hand:
          if (active_r()) {
              cairo_set_source_rgb(cr,61.0/255,61.0/255,61.0/255);
          } else {
              cairo_set_source_rgb(cr,111.0/255,111.0/255,111.0/255);
          }
          cairo_rotate(cr,val*3/2*M_PI+0.25*M_PI);
          cairo_set_line_width (cr, 2.3);
          cairo_move_to(cr,0,0);
          cairo_line_to(cr,0,d*rHand);
          cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
          cairo_stroke (cr);
          //freeing the resources
          cairo_pattern_destroy(pat);
          cairo_surface_destroy(Xsurface);  cairo_destroy(cr);
}

void WidgetPDial::pdialcolor(int r,int g,int b) {
  if (active_r())
              fl_color(r, g, b);
          else
              fl_color(160 - (160 - r) / 3, 160 - (160 - b) / 3, 160 - (160 - b) / 3);
}

void WidgetPDial::setValueType(ValueType type_) {
  valueType = type_;
            if(type_ == VC_FilterVelocitySense)
                tipwin->setGraphicsValueType(type_);
            tipwin->customValue(convert_value(valueType,value()));
}

int WidgetPDial::value(double v) {
  int ret = Fl_Valuator::value(v);
            tipwin->value(v); // set value for graphics
            tipwin->customValue(convert_value(valueType,v));
            return(ret);
}

double WidgetPDial::value() {
  return Fl_Valuator::value();
}

void WidgetPDial::value_cb2() {
  tipwin->customValue(convert_value(valueType,value()));
}

void WidgetPDial::value_cb(Fl_Widget*, void*data) {
  WidgetPDial *val = (WidgetPDial*)data;
          val->value_cb2();
}

void WidgetPDial::tooltip(const char * c) {
  tipwin->setText(c);
          textset = true;
}
